unit IAMenuContent;

interface
uses Classes,graphics;

type
TIAGhangedProcedure = procedure of object;
TIABarPosition = (iabpMain,iabpAlternate1,iabpAlternate2);

TIAmenuList = class;
TIAmenuItem = class(TPersistent)
private
  FItemName : string;
  FSelected : Boolean;
  FSplitter : Boolean;
  FTransparent : Boolean;
  FGrayed   : Boolean;
  FHidden   : Boolean;
  FStretch  : Boolean;
  FContinued : Boolean;
  FSubMenuList : TIAmenuList;
  FOnChange : TIAGhangedProcedure;
  FBackBitmap : TBitmap;
  FMainBitmap : TBitmap;



  procedure ReadSubBMenu(Stream : TStream);



  procedure WriteSubBMenu(Stream : TStream);

  procedure ReadBackBitmap(Stream : TStream);
  procedure WriteBackBitmap(Stream : TStream);
  procedure ReadMainBitmap(Stream : TStream);
  procedure WriteMainBitmap(Stream : TStream);


  procedure WriteBinaryData(Stream : Tstream);

  procedure ReadBinaryData(Stream : TStream);

protected
  procedure SetItemName(Value : String);virtual;
  procedure SetSelected(Value : Boolean);virtual;
  procedure SetGrayed(Value : Boolean);virtual;
  procedure SetHidden(Value : Boolean);virtual;
  procedure SetSplitter(Value : Boolean);virtual;
  procedure SetTransparent(Value : Boolean);virtual;
  procedure SetStretch(Value : Boolean);virtual;
  procedure SetContinued(Value : Boolean);virtual;
  procedure SetSubMenuList(Value : TIAmenuList);virtual;
  procedure SetBackBitmap(Value : Tbitmap);virtual;
  procedure SetMainBitmap(Value : TBitmap);virtual;
  procedure SetOnChange(Value : TIAGhangedProcedure);virtual;

  procedure DefineProperties(Filer: TFiler); override;

public
  Owner : TPersistent;
  procedure Assign(Source: TPersistent); override;
  constructor Create(AOwner: TPersistent); virtual;
  destructor Destroy; override;

  property ItemName : string read FItemName write SetItemName;
  property Selected : Boolean read FSelected write SetSelected default false;
  property Grayed : Boolean read FGrayed write SetGrayed default false;
  property Hidden : Boolean read FHidden write SetHidden default false;
  property Splitter : Boolean read FSplitter write SetSplitter default false;
  property Transparent : Boolean read FTransparent write SetTransparent default false;
  property Stretch : Boolean read FStretch write SetStretch default false;
  property Continued : Boolean read FContinued write SetContinued default true;
  property SubMenuList : TIAmenuList read FSubMenuList write SetSubMenuList default nil;
  property OnChange : TIAGhangedProcedure read FOnChange write SetOnChange default nil;
  property BackBitmap : TBitmap read FBackBitmap write SetBackBitmap default nil;
  property MainBitmap : TBitmap read FMainBitmap write SetMainBitmap default nil;  

end;
TIAItemsArray = array of TIAmenuItem;
TIAmenuList = class(TPersistent)
private
  FbackBitmap : TBitmap;
  FShowHidden : Boolean;
  FListContinued : Boolean;
  FMainBackStretch : Boolean;
  FItems : TIAItemsArray;
  FOnChange : TIAGhangedProcedure;
  FBarStretch: Boolean;
  FBarTransparent: Boolean;
  FBarText: String;
  FBarBackBitmap: TBitmap;
  FBarMainBitmap: TBitmap;
  FBarFont: TFont;
  FBarPosition: TIABarPosition;

  procedure ReadBItemss(Stream : TStream);
  procedure WriteBItemss(Stream : TStream);
  procedure ReadBackBitmap(Stream : TStream);
  procedure WriteBackBitmap(Stream : TStream);
  procedure ReadBarBackBitmap(Stream: TStream);
  procedure ReadBarMainBitmap(Stream: TStream);
  procedure WriteBarBackBitmap(Stream: TStream);
  procedure WriteBarMainBitmap(Stream: TStream);

  procedure WriteBinaryContent(Stream : TStream);
  procedure ReadBinaryContent(Stream : TStream);


protected
  procedure DefineProperties(Filer: TFiler); override;
  function GetItemsNumber: integer; virtual;
  function GetItems(Index : integer): TIAmenuItem;virtual;
  procedure SetItems(Index : integer;Value : TIAmenuItem);virtual;
  procedure SetBackBitmap(Value : TBitmap);virtual;
  procedure SetShowHidden(Value : Boolean);virtual;
  procedure SetMainBackStretch(Value : Boolean);virtual;
  procedure SetOnChange(Value : TIAGhangedProcedure);virtual;
  procedure SetListContinued(Value: boolean);virtual;
  procedure SetBarBackBitmap(Value: TBitmap);virtual;
  procedure SetBarFont(Value: TFont);virtual;
  procedure SetBarMainBitmap(Value: TBitmap);virtual;
  procedure SetBarStretch(Value: Boolean); virtual;
  procedure SetBarText(Value: String);virtual;
  procedure SetBarTransparent(Value: Boolean);virtual;
  procedure SetBarPosition(Value: TIABarPosition);virtual;
public
  Owner : TPersistent;
  procedure Assign(Source: TPersistent); override;
  constructor Create(AOwner: Tpersistent); virtual;
  destructor  Destroy; override;

  procedure AddItem;
  procedure DeleteItem(Index : integer);
  procedure InsertItem(BeforeIndex : integer);
  procedure SwapItems(Index1,Index2 : integer);
  property Items[Index : integer]: TIAmenuItem read GetItems write SetItems; default;
  property ItemsNumber : integer read GetItemsNumber;
  property BackBitmap : TBitmap read FBackBitmap write SetBackBitmap default nil;
  property ShowHidden : Boolean read FShowHidden write SetShowHidden default false;
  property MainBackStretch : Boolean read FMainBackStretch write SetMainBackStretch default false;
  property ListContinued : boolean read FListContinued write SetListContinued default true;
  property OnChange : TIAGhangedProcedure read FOnChange write SetOnChange default nil;
  property BarFont : TFont read FBarFont write SetBarFont;
  property BarText : String read FBarText write SetBarText;
  property BarBackBitmap : TBitmap read FBarBackBitmap write SetBarBackBitmap default nil;
  property BarMainBitmap : TBitmap read FBarMainBitmap write SetBarMainBitmap default nil;
  property BarTransparent : Boolean read FBarTransparent write SetBarTransparent default false;
  property BarStretch : Boolean read FBarStretch write SetBarStretch default false;
  property BarPosition : TIABarPosition read FBarPosition write SetBarPosition default iabpMain;
end;

implementation
  //TIAmenuItem...
constructor TIAmenuItem.Create(AOwner: TPersistent);
begin
  inherited Create;
  Owner:=AOwner;
  FItemName:='';
  FSelected:=false;
  FGrayed:=false;
  FHidden:=false;
  FSplitter:=false;
  FTransparent:=false;
  FStretch:=false;
  FContinued:=TRUE;
  FSubMenuList:=nil;
  FOnChange:=nil;
  FBackBitmap:=nil;
  FMainBitmap:=nil;
end;
destructor TIAmenuItem.Destroy;
begin
  if FSubMenuList<>nil then FSubMenuList.Free;
  if FBackBitmap<>nil then FBackBitmap.free;
  if FMainBitmap<>nil then FMainBitmap.free;
  inherited Destroy;
end;
procedure TIAmenuItem.SetItemName(Value : String);
begin
  if Value<>FItemName then
    begin
      FItemName:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;

procedure TIAmenuItem.SetSelected(Value : Boolean);
begin
  if Value<>FSelected then
    begin
      FSelected:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetGrayed(Value : Boolean);
begin
  if Value<>FGrayed then
    begin
      FGrayed:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetHidden(Value : Boolean);
begin
  if Value<>FHidden then
    begin
      FHidden:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetSplitter(Value : Boolean);
begin
  if Value<>FSplitter then
    begin
      FSplitter:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetTransparent(Value : Boolean);
begin
  if Value<>FTransparent then
    begin
      FTransparent:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetStretch(Value : Boolean);
begin
  if Value<>FStretch then
    begin
      FStretch:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetContinued(Value: Boolean);
begin
  if Value<>FContinued then
    begin
      FContinued:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetSubMenuList(Value : TIAmenuList);
begin
  if Value<>FSubMenuList then
    begin
      if FSubMenuList=nil then
        begin
          FSubMenuList:=TIAmenuList.Create(self);
          FSubMenuList.Assign(Value);
        end else
        begin
          if Value=nil then
            begin
              FSubMenuList.Free;
              FSubMenuList:=nil;
            end else
            begin
              FSubMenuList.Assign(Value);
            end;
        end;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuItem.SetBackBitmap(Value : Tbitmap);
begin
  if Value=nil then
    begin
      if FBackBitmap<>nil then FBackBitmap.free;
      FBackBitmap:=nil;
    end else
    begin
      if FBackBitmap<>nil then
        begin
          FBackBitmap.Assign(Value);
        end else
        begin
          FBackBitmap:=TBitmap.Create;
          FBackBitmap.Assign(Value);
        end;
    end;
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuItem.SetMainBitmap(Value : TBitmap);
begin
  if Value=nil then
    begin
      if FMainBitmap<>nil then FMainBitmap.free;
      FMainBitmap:=nil;
    end else
    begin
      if FMainBitmap<>nil then
        begin
          FMainBitmap.Assign(Value);
        end else
        begin
          FMainBitmap:=TBitmap.Create;
          FMainBitmap.Assign(Value);
        end;
    end;
  if assigned(FOnChange) then FonChange;  
end;
procedure TIAmenuItem.SetOnChange(Value : TIAGhangedProcedure);
begin
      FOnChange:=Value;
      if FSubMenuList<>nil then FSubMenuList.OnChange:=Value;
end;
procedure TIAmenuItem.Assign(Source: TPersistent);
begin
//  inherited Assign(Source);
  if Source is TIAmenuItem then
    begin
      FItemName:=TIAmenuItem(Source).FItemName;
      FSelected:=TIAmenuItem(Source).FSelected;
      FGrayed:=TIAmenuItem(Source).FGrayed;
      FHidden:=TIAmenuItem(Source).FHidden;
      FTransparent:=TIAmenuItem(Source).FTransparent;
      FSplitter:=TIAmenuItem(Source).FSplitter;
      FStretch:=TIAmenuItem(Source).FStretch;
      FContinued:=TIAmenuItem(Source).FContinued;
      FOnChange:=nil;
      SubMenuList:=TIAmenuItem(Source).FSubMenuList;
      BackBitmap:=TIAmenuItem(Source).FBackBitmap;
      MainBitmap:=TIAmenuItem(Source).FMainBitmap;
      FOnChange:=TIAmenuItem(Source).FOnChange;
      if assigned(FOnChange) then FonChange;
    end;
end;

procedure TIAmenuItem.WriteBinaryData(Stream : Tstream);
var
  i : integer;

begin
  WriteBackBitmap(Stream);
  WriteMainBitmap(Stream);
  if FSelected then i:=1 else i:=0;
  Stream.Write(i,4);
  if FSplitter then i:=1 else i:=0;
  Stream.Write(i,4);
  if FTransparent then i:=1 else i:=0;
  Stream.Write(i,4);
  if FGrayed then i:=1 else i:=0;
  Stream.Write(i,4);
  if FHidden then i:=1 else i:=0;
  Stream.Write(i,4);
  if FStretch then i:=1 else i:=0;
  Stream.Write(i,4);
  if FContinued then i:=1 else i:=0;
  Stream.Write(i,4);


  i:=length(FItemName);
  Stream.Write(i,4);
  if i<>0 then Stream.Write(FItemName[1],length(FItemName));

  {FItemName : string;
  FSelected : Boolean;
  FSplitter : Boolean;
  FTransparent : Boolean;
  FGrayed   : Boolean;
  FHidden   : Boolean;
  FStretch  : Boolean;
  FSubMenuList : TIAmenuList;
  FOnChange : TIAGhangedProcedure;
  FBackBitmap : TBitmap;
  FMainBitmap : TBitmap;}



  WriteSubBMenu(Stream);
end;

procedure TIAmenuItem.ReadBinaryData(Stream : TStream);
var
  i : integer;
  S : String;
begin
  ReadBackBitmap(Stream);
  ReadMainBitmap(Stream);
  Stream.Read(i,4);
  if i=0 then Selected:=false else Selected:=TRUE;
  Stream.Read(i,4);
  if i=0 then Splitter:=false else Splitter:=TRUE;
  Stream.Read(i,4);
  if i=0 then Transparent:=false else Transparent:=TRUE;
  Stream.Read(i,4);
  if i=0 then Grayed:=false else Grayed:=TRUE;
  Stream.Read(i,4);
  if i=0 then Hidden:=false else Hidden:=TRUE;
  Stream.Read(i,4);
  if i=0 then Stretch:=false else Stretch:=TRUE;
  Stream.Read(i,4);
  if i=0 then Continued:=false else Continued:=TRUE;
  Stream.Read(i,4);
  if i<>0 then
    begin
      setlength(s,i);
      Stream.Read(s[1],i);
      FItemName:=s;
    end;
  {FItemName : string;
  FSelected : Boolean;
  FSplitter : Boolean;
  FTransparent : Boolean;
  FGrayed   : Boolean;
  FHidden   : Boolean;
  FStretch  : Boolean;
  FSubMenuList : TIAmenuList;
  FOnChange : TIAGhangedProcedure;
  FBackBitmap : TBitmap;
  FMainBitmap : TBitmap;}
  ReadSubBMenu(Stream);
end;
procedure TIAmenuItem.DefineProperties(Filer: TFiler);
begin
//
end;
procedure TIAmenuItem.ReadBackBitmap(Stream : TStream);
var I: Byte;
 B1 : TBitmap;

begin

 Stream.Read(i,1);
 if i<>0 then
   begin

     B1:=TBitmap.Create;
     B1.LoadFromStream(Stream);
     BackBitmap:=B1;
     B1.free;
   end;
end;
procedure TIAmenuItem.WriteBackBitmap(Stream : TStream);
var i : Byte;

begin

  if self.FbackBitmap=nil then i:=0 else i:=1;
  Stream.Write(I,1);
  if i<>0 then
    begin

      FBackBitmap.SaveToStream(Stream);
    end;
end;
procedure TIAmenuItem.ReadMainBitmap(Stream : TStream);
var I: Byte;
 B1 : TBitmap;

begin
 Stream.Read(i,1);
 if i<>0 then
   begin

     B1:=TBitmap.Create;
     B1.LoadFromStream(Stream);
     MainBitmap:=B1;
     B1.free;
   end;
end;
procedure TIAmenuItem.WriteMainBitmap(Stream : TStream);
var i : Byte;

begin

  if self.FMainBitmap=nil then i:=0 else i:=1;
  Stream.Write(I,1);
  if i<>0 then
    begin

      FMainBitmap.SaveToStream(Stream);
    end;
end;


procedure TIAmenuItem.ReadSubBMenu(Stream : TStream);
var i : Byte;
    TL : TIAmenuList;

begin
  Stream.Read(i,1);
  if i<>0 then
    begin
      if   self.SubMenuList=nil then
        begin
          TL:=TIAmenuList.Create(self);
          TL.ReadBinaryContent(Stream);
          self.SubMenuList:=TL;
          TL.Free;
        end else
        begin
          self.SubMenuList.ReadBinaryContent(Stream);
        end;

    end;
end;

procedure TIAmenuItem.WriteSubBMenu(Stream : TStream);
var i : byte;
begin

  if  self.SubMenuList<>nil then i:=1 else i:=0;
  Stream.Write(i,1);
  if i<>0 then  self.SubMenuList.WriteBinaryContent(Stream);

end;



//---------------TIAmenuList
procedure TIAmenuList.Assign(Source: TPersistent);
var i : integer;
begin
//  inherited Assign(Source);
  if Source is  TIAmenuList then
    begin
      FOnChange:=nil;
      FShowHidden:=TIAMenuList(Source).FShowHidden;
      FMainBackStretch:=TIAMenuList(Source).FMainBackStretch;
      FListContinued:=TIAMenuList(Source).FListContinued;
      FBarStretch:=TIAMenuList(Source).FBarStretch;
      FBarTransparent:=TIAMenuList(Source).FBarTransparent;
      FBarPosition:=TIAMenuList(Source).FBarPosition;
      FBarText:=TIAMenuList(Source).FBarText;

      BarFont:=TIAMenuList(Source).BarFont;
      backBitmap:=TIAMenuList(Source).BackBitmap;
      BarBackBitmap:=TIAMenuList(Source).BarBackBitmap;
      BarMainBitmap:=TIAMenuList(Source).BarMainBitmap;
      for i:=0 to ItemsNumber-1 do
        begin
          FItems[i].free;
        end;
      seTlength(FItems,TIAMenuList(Source).itemsNumber);
      for i:=0 to ItemsNumber-1 do
        begin
          FItems[i]:=TIAMenuItem.Create(self);
          FItems[i].assign(TIAMenuList(Source).Items[i]);
        end;
      if assigned(FOnChange) then FonChange;
    end;
end;
constructor TIAmenuList.Create(AOwner: TPersistent);
begin
  inherited Create;
  Owner:=Aowner;
  FonChange:=nil;
  FShowHidden:=false;
  FMainBackStretch:=false;
  FListContinued:=true;
  FBackBitmap:=nil;
  FBarStretch:=false;
  FBarTransparent:=false;
  FBarText:='';
  FBarBackBitmap:=nil;
  FBarMainBitmap:=nil;
  FBarFont:=TFont.Create;
  FBarFont.Name:='Times New Roman';
  FBarPosition:=iabpMain;
end;
destructor TIAmenuList.Destroy;
var i : integer;
begin
  for i:=0 to ItemsNumber-1 do
    begin
      FItems[i].free;
    end;
  FBarFont.Free;  
  if FBackBitmap<>nil then FBackBitmap.free;
  if FBarBackBitmap<>nil then FBarBackBitmap.free;
  if FBarMainBitmap<>nil then FBarMainBitmap.free;
  inherited Destroy;
end;
function TIAmenuList.GetItems(Index : integer): TIAmenuItem;
begin
  if (Index<length(FItems)) and (Index>=0) then
  Result:=FItems[Index] else Result:=FItems[0];
end;
procedure TIAmenuList.SetItems(Index : integer;Value : TIAmenuItem);
begin
  FItems[Index].assign(Value);
end;
function TIAmenuList.GetItemsNumber: integer;
begin
  Result:=length(FItems);
end;
procedure TIAmenuList.SetBackBitmap(Value : Tbitmap);
begin
  if Value=nil then
    begin
      if FBackBitmap<>nil then FBackBitmap.free;
      FBackBitmap:=nil;
    end else
    begin
      if FBackBitmap<>nil then
        begin
          FBackBitmap.Assign(Value);
        end else
        begin
          FBackBitmap:=TBitmap.Create;
          FBackBitmap.Assign(Value);
        end;
    end;
  if assigned(FOnChange) then FonChange;
end;

procedure TIAmenuList.SetShowHidden(Value : Boolean);
begin
  if Value<>FShowHidden then
    begin
      FShowHidden:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuList.SetMainBackStretch(Value : Boolean);
begin
  if Value<>FMainBackStretch then
    begin
      FMainBackStretch:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuList.SetListContinued(Value: boolean);
begin
  if Value<>FListContinued then
    begin
      FListContinued:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
//----
procedure TIAmenuList.SetBarFont(Value: TFont);
begin
  if Value<>nil then
    begin
      FBarFont.Assign(Value);
      if assigned(FOnChange) then FonChange;
    end;
end;

procedure TIAmenuList.SetBarMainBitmap(Value: TBitmap);
begin
  if Value=nil then
    begin
      if FBarMainBitmap<>nil then FBarMainBitmap.free;
      FBarMainBitmap:=nil;
    end else
    begin
      if FBarMainBitmap<>nil then
        begin
          FBarMainBitmap.Assign(Value);
        end else
        begin
          FBarMainBitmap:=TBitmap.Create;
          FBarMainBitmap.Assign(Value);
        end;
    end;
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuList.SetBarBackBitmap(Value: TBitmap);
begin
  if Value=nil then
    begin
      if FBarBackBitmap<>nil then FBarBackBitmap.free;
      FBarBackBitmap:=nil;
    end else
    begin
      if FBarBackBitmap<>nil then
        begin
          FBarBackBitmap.Assign(Value);
        end else
        begin
          FBarBackBitmap:=TBitmap.Create;
          FBarBackBitmap.Assign(Value);
        end;
    end;
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuList.SetBarStretch(Value: Boolean);
begin
  if Value<>FBarStretch then
    begin
      FBarStretch:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;

procedure TIAmenuList.SetBarText(Value: String);
begin
  if Value<>FBarText then
    begin
      FBarText:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;

procedure TIAmenuList.SetBarTransparent(Value: Boolean);
begin
  if Value<>FBarTransparent then
    begin
      FBarTransparent:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
procedure TIAmenuList.SetBarPosition(Value: TIABarPosition);
begin
  if Value<>FBarPosition then
    begin
      FBarPosition:=Value;
      if assigned(FOnChange) then FonChange;
    end;
end;
//----
procedure TIAmenuList.SetOnChange(Value : TIAGhangedProcedure);
var i : integer;
begin
  FOnChange:=Value;
  for i:=0 to Itemsnumber-1 do
    begin
      self.Items[i].OnChange:=Value;
    end;
end;
procedure TIAmenuList.AddItem;
var
  TempArray : TIAItemsArray;
  i : integer;
begin
  setlength(TempArray,length(FItems));
  for i:=0 to length(FItems)-1 do
    begin
      TempArray[i]:=FItems[i];
    end;
  setlength(FItems,length(Fitems)+1);
  for i:=0 to length(FItems)-2 do
    begin
      FItems[i]:=TempArray[i];
    end;
  FItems[length(FItems)-1]:=TIAmenuItem.Create(self);
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuList.DeleteItem(Index : integer);
var
  TempArray : TIAItemsArray;
  i : integer;
begin
  setlength(TempArray,length(FItems));
  for i:=0 to length(FItems)-1 do
    begin
      TempArray[i]:=FItems[i];
    end;
  setlength(FItems,length(Fitems)-1);
  for i:=0 to Index-1 do
    begin
      FItems[i]:=TempArray[i];
    end;
  for i:=Index+1 to length(FItems) do
    begin
      FItems[i-1]:=TempArray[i];
    end;
  TempArray[Index].free;
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuList.InsertItem(BeforeIndex : integer);
var
  TempArray : TIAItemsArray;
  i : integer;
begin
  setlength(TempArray,length(FItems)+1);
  for i:=0 to BeforeIndex-1 do
    begin
      TempArray[i]:=FItems[i];
    end;
  TempArray[BeforeIndex]:=TIAmenuItem.Create(self);
  for i:=BeforeIndex to length(FItems)-1 do
    begin
      TempArray[i+1]:=FItems[i];
    end;
  setlength(FItems,length(Fitems)+1);
  for i:=0 to length(FItems)-1 do
    begin
      FItems[i]:=TempArray[i];
    end;
  if assigned(FOnChange) then FonChange;
end;
procedure TIAmenuList.SwapItems(Index1,Index2 : integer);
var
  A1 : TIAmenuItem;
begin
  A1:=FItems[Index1];
  FItems[Index1]:=FItems[Index2];
  FItems[Index2]:=A1;
  if assigned(FOnChange) then FonChange;
end;


procedure TIAmenuList.DefineProperties(Filer: TFiler);
begin
  Filer.DefineBinaryProperty('Content',ReadBinaryContent,WriteBinaryContent,TRUE);
end;



procedure TIAmenuList.ReadBItemss(Stream : TStream);
var
  N,i : integer;
begin
  Stream.Read(N,4);
  if self.ItemsNumber<>N
    then
    begin
      for i:=0 to self.ItemsNumber-1 do
      self.DeleteItem(0);
      for i:=0 to N-1 do
      self.AddItem;
    end;

  for i:=0 to N-1 do
    self.Items[i].ReadBinaryData(Stream);

end;
procedure TIAmenuList.WriteBItemss(Stream : TStream);
var
 i,N : integer;
begin
  N:=self.ItemsNumber;
  Stream.Write(N,4);
  for i:=0 to self.ItemsNumber-1 do
    self.Items[i].WriteBinaryData(Stream);

end;
procedure TIAmenuList.ReadBackBitmap(Stream : TStream);
var I: Byte;
 B1 : TBitmap;
begin
 Stream.Read(i,1);
 if i<>0 then
   begin
     B1:=TBitmap.Create;
     B1.LoadFromStream(Stream);
     BackBitmap:=B1;
     B1.free;
   end;
end;
procedure TIAmenuList.ReadBarBackBitmap(Stream : TStream);
var I: Byte;
 B1 : TBitmap;
begin
 Stream.Read(i,1);
 if i<>0 then
   begin
     B1:=TBitmap.Create;
     B1.LoadFromStream(Stream);
     BarBackBitmap:=B1;
     B1.free;
   end;
end;
procedure TIAmenuList.ReadBarMainBitmap(Stream : TStream);
var I: Byte;
 B1 : TBitmap;
begin
 Stream.Read(i,1);
 if i<>0 then
   begin
     B1:=TBitmap.Create;
     B1.LoadFromStream(Stream);
     BarMainBitmap:=B1;
     B1.free;
   end;
end;
procedure TIAmenuList.WriteBackBitmap(Stream : TStream);
var i : Byte;
begin
  if self.FbackBitmap=nil then i:=0 else i:=1;
  Stream.Write(I,1);
  if i<>0 then
    begin
      FBackBitmap.SaveToStream(Stream);
    end;
end;
procedure TIAmenuList.WriteBarBackBitmap(Stream : TStream);
var i : Byte;
begin
  if self.FBarBackBitmap=nil then i:=0 else i:=1;
  Stream.Write(I,1);
  if i<>0 then
    begin
      FBarBackBitmap.SaveToStream(Stream);
    end;
end;
procedure TIAmenuList.WriteBarMainBitmap(Stream : TStream);
var i : Byte;
begin
  if self.FBarMainBitmap=nil then i:=0 else i:=1;
  Stream.Write(I,1);
  if i<>0 then
    begin
      FBarMainBitmap.SaveToStream(Stream);
    end;
end;
procedure TIAmenuList.WriteBinaryContent(Stream: TStream);
var
  i : integer;
  S : String;
  AColor : TColor;
begin
//
  WriteBackBitmap(Stream);
  WriteBarBackBitmap(Stream);
  WriteBarMainBitmap(Stream);
  if MainBackStretch then i:=1 else i:=0;
  Stream.Write(i,4);
  if ShowHidden      then i:=1 else i:=0;
  Stream.Write(i,4);
  if ListContinued   then i:=1 else i:=0;
  Stream.Write(i,4);
  if FBarTransparent   then i:=1 else i:=0;
  Stream.Write(i,4);
  if FBarStretch       then i:=1 else i:=0;
  Stream.Write(i,4);
  Stream.Write(FBarPosition,4);
  i:=length(FBarText);
  Stream.Write(i,4);
  if i<>0 then Stream.Write(FBarText[1],length(FBarText));
  S:=FBarFont.Name;
  i:=length(S);
  Stream.Write(i,4);
  if i<>0 then Stream.Write(S[1],length(S));
  AColor:=FBarFont.Color;
  Stream.Write(AColor,4);
  i:=FBarFont.Size;
  Stream.Write(i,4);
  if fsBold in FBarFont.Style then i:=1 else i:=0;
  Stream.Write(i,4);
  if fsItalic in FBarFont.Style then i:=1 else i:=0;
  Stream.Write(i,4);
  if fsUnderline in FBarFont.Style then i:=1 else i:=0;
  Stream.Write(i,4);
  if fsStrikeout in FBarFont.Style then i:=1 else i:=0;
  Stream.Write(i,4);


  WriteBItemss(Stream);
end;

procedure TIAmenuList.ReadBinaryContent(Stream: TStream);
var
  i : integer;
  S : String;
  AColor : TColor;
begin
//
  ReadBackBitmap(Stream);
  ReadBarBackBitmap(Stream);
  ReadBarMainBitmap(Stream);
  Stream.Read(i,4);
  if i=0 then MainBackStretch:=false    else MainBackStretch:=TRUE;
  Stream.Read(i,4);
  if i=0 then ShowHidden:=false         else ShowHidden:=TRUE;
  Stream.Read(i,4);
  if i=0 then ListContinued:=false      else ListContinued:=TRUE;
  Stream.Read(i,4);
  if i=0 then BarTransparent:=false      else BarTransparent:=TRUE;
  Stream.Read(i,4);
  if i=0 then BarStretch:=false      else BarStretch:=TRUE;
  Stream.Read(FBarPosition,4);
  Stream.Read(i,4);
  if i<>0 then
    begin
      setlength(s,i);
      Stream.Read(s[1],i);
      FBarText:=s;
    end;
  Stream.Read(i,4);  
  if i<>0 then
    begin
      setlength(s,i);
      Stream.Read(s[1],i);
      FBarFont.Name:=s;
    end;
  Stream.Read(AColor,4);
  FBarFont.Color:=AColor;
  Stream.Read(i,4);
  FBarFont.Size:=i;
  FBarFont.Style:=[];
  Stream.Read(i,4);
  if i<>0 then FBarFont.Style:=FBarFont.Style+[fsBold];
  Stream.Read(i,4);
  if i<>0 then FBarFont.Style:=FBarFont.Style+[fsItalic];
  Stream.Read(i,4);
  if i<>0 then FBarFont.Style:=FBarFont.Style+[fsUnderline];
  Stream.Read(i,4);
  if i<>0 then FBarFont.Style:=FBarFont.Style+[fsStrikeout];

  ReadBItemss(Stream);
end;

end.
